package ctbrec.ui.sites.secretfriends;

import ctbrec.Config;
import ctbrec.Model;
import ctbrec.io.HtmlParser;
import ctbrec.io.HttpException;
import ctbrec.sites.secretfriends.SecretFriends;
import ctbrec.sites.secretfriends.SecretFriendsModelParser;
import ctbrec.ui.SiteUiFactory;
import ctbrec.ui.tabs.PaginatedScheduledService;
import javafx.concurrent.Task;
import okhttp3.Request;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.*;

import static ctbrec.io.HttpConstants.*;

public class SecretFriendsUpdateService extends PaginatedScheduledService {

    private static final Logger LOG = LoggerFactory.getLogger(SecretFriendsUpdateService.class);

    private final String url;
    private final boolean loginRequired;
    private final SecretFriends site;

    public SecretFriendsUpdateService(String url, boolean loginRequired, SecretFriends site) {
        this.url = url;
        this.loginRequired = loginRequired;
        this.site = site;
    }

    @Override
    protected Task<List<Model>> createTask() {
        return new Task<>() {
            @Override
            public List<Model> call() throws IOException {
                if (loginRequired && !site.credentialsAvailable()) {
                    return Collections.emptyList();
                } else {
                    String paginatedUrl = url + (url.indexOf('?') >= 0 ? '&' : '?') + "Friend_page=" + page;
                    LOG.debug("Fetching page {}", paginatedUrl);
                    if (loginRequired) {
                        SiteUiFactory.getUi(site).login();
                    }
                    var request = new Request.Builder()
                            .url(paginatedUrl)
                            .header(ACCEPT, MIMETYPE_APPLICATION_JSON)
                            .header(ACCEPT_LANGUAGE, Locale.ENGLISH.getLanguage())
                            .header(X_REQUESTED_WITH, XML_HTTP_REQUEST)
                            .header(USER_AGENT, Config.getInstance().getSettings().httpUserAgent)
                            .build();
                    try (var response = site.getHttpClient().execute(request)) {
                        if (response.isSuccessful()) {
                            return parseModels(Objects.requireNonNull(response.body(), "HTTP response body is null").string());
                        } else {
                            throw new HttpException(response.code(), response.message());
                        }
                    }
                }

            }
        };
    }

    private List<Model> parseModels(String body) {
        List<Model> models = new ArrayList<>();
        Elements modelDivs = HtmlParser.getTags(body, "div[class~=model-wrapper]");
        LOG.debug("Found {} models", modelDivs.size());
        for (Element div : modelDivs) {
            try {
                models.add(SecretFriendsModelParser.parse(site, div));
            } catch (Exception e) {
                LOG.warn("Couldn't parse one of the models: {}", div.html(), e);
            }
        }
        return models;
    }


}
